/*
 * This file is part of the Ant Control Language compiler developed by
 * the team "KaNT" during the ICFP 2004 programming contest.
 *
 * ----------------------------------------------------------------------
 * 
 * Copyright (C) 2004  the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke
 * Thomas Moschny
 * Bernhard Haumacher
 * Bjoern-Oliver Hartmann
 *
 * ----------------------------------------------------------------------
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA 02111-1307, USA.
 *
 */

package ant.compiler.asm;

import java.util.HashMap;
import java.util.Iterator;


/**
 * Stores the connection between jumps and labeled {@link Asm}
 * statements. This information is produced during {@link
 * ant.compiler.gen.CodeGenerator code generation}.
 * 
 * <p>License: GPL</p>
 *
 * @author <a href="http://www.haumacher.de/>">Bernhard Haumacher</a>
 */
public class AsmSymbols {
    HashMap labeledOp = new HashMap();
    HashMap jumpName = new HashMap();

    public void registerLabel(String name, Asm target) {
	labeledOp.put(name, target);
    }

    public void registerJump(Asm jump, String name) {
	jumpName.put(jump, name);
    }

    public Asm getTarget(String name) {
	return (Asm) labeledOp.get(name);
    }

    public String getJumpLabel(Asm jump) {
	return (String) jumpName.get(jump);
    }

    public Iterator getJumps() {
	return jumpName.keySet().iterator();
    }

    public void resolveLabels() {
	for (Iterator ops = getJumps(); ops.hasNext(); ) {
	    Asm op = (Asm) ops.next();

	    String name = getJumpLabel(op);
	    Asm target = getTarget(name);

	    // System.err.println("resolving jump: op=" + op + 
	    //     ", label=" + name + ", target=" + target);

	    op.setNext(target);
	}
    }
}
