/*
 * This file is part of the Ant Control Language compiler developed by
 * the team "KaNT" during the ICFP 2004 programming contest.
 *
 * ----------------------------------------------------------------------
 * 
 * Copyright (C) 2004  the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke
 * Thomas Moschny
 * Bernhard Haumacher
 * Bjoern-Oliver Hartmann
 *
 * ----------------------------------------------------------------------
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA 02111-1307, USA.
 *
 */

package ant.compiler.asm;

import ant.compiler.asm.Asm.*;
import ant.compiler.util.ErrorLog;


/**
 * Computes {@link Asm.Nop#replacement replacements} for {@link
 * Asm.Nop} statements.
 *
 * <p>License: GPL</p>
 *
 * @author <a href="http://www.haumacher.de/>">Bernhard Haumacher</a>
 */
public class NopAnalyzer extends Asm.DFSPostorderVisitor {
    ErrorLog log;
    Asm result;

    public NopAnalyzer(ErrorLog log) {
	this.log = log;
    }

    public void visitSequential(Sequential op) {
	if (! visited(op)) {
	    markVisited(op);
	    op.next.visit(this);
	    op.next = result;
	}

	result = op;
    }

    public void visitConditional(Conditional op) {
	if (! visited(op)) {
	    markVisited(op);
	    op.nextThen.visit(this);
	    op.nextThen = result;

	    op.nextElse.visit(this);
	    op.nextElse = result;
	}
	result = op;
    }

    public void visit(Nop op) {
	if (visited(op)) return;

	if (op.next == null) {
	    log.error("Missing next statement");
	    return;
	}

	result = null;
	op.next.visit(this);
	op.replacement = result;

	if (result == null) log.error("Unreplaceable nop");
    }
}
