/*
 * This file is part of the Ant Control Language compiler developed by
 * the team "KaNT" during the ICFP 2004 programming contest.
 *
 * ----------------------------------------------------------------------
 * 
 * Copyright (C) 2004  the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke
 * Thomas Moschny
 * Bernhard Haumacher
 * Bjoern-Oliver Hartmann
 *
 * ----------------------------------------------------------------------
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA 02111-1307, USA.
 *
 */

package ant.compiler.asm;

import ant.compiler.asm.Asm.*;
import ant.compiler.util.ErrorLog;


/**
 * Eliminates {@link Asm.Nop} statements from a graph of {@link Asm}
 * statements.
 *
 * <p>License: GPL</p>
 *
 * @author <a href="http://www.haumacher.de/>">Bernhard Haumacher</a>
 */
public class NopEliminator extends Asm.DFSPostorderVisitor {
    ErrorLog log;
    Asm result;

    public NopEliminator(ErrorLog log) {
	this.log = log;
    }

    public Asm getResult() {
	return result;
    }

    public void visitAsm(Asm op) {
	super.visitAsm(op);
	result = op;
    }

    public void visit(Nop op) {
	super.visit(op);
	result = op.replacement;
    }

    public void visit(Move op) {
	if (! visited(op)) {
	    if (op.nextThen != op.nextElse) 
		throw new AssertionError("conditional move before resolution");

	    if (op.nextThen instanceof NopWallTest) {
		op.nextThen = ((NopWallTest) op.nextThen).nextThen;
		op.nextElse = ((NopWallTest) op.nextElse).nextElse;
	    }
	}
	super.visit(op);
    }

    public void visit(PickUp op) {
	if (! visited(op)) {
	    if (op.nextThen != op.nextElse) 
		throw new AssertionError("conditional pickup before resolution");

	    if (op.nextThen instanceof NopFoodTest) {
		op.nextThen = ((NopFoodTest) op.nextThen).nextThen;
		op.nextElse = ((NopFoodTest) op.nextElse).nextElse;
	    }
	}	
	super.visit(op);
    }

    public void visit(NopWallTest op) {
	super.visit(op);
	log.error("unresolved wall test");
    }

    public void visit(NopFoodTest op) {
	super.visit(op);
	log.error("unresolved food test");
    }
}
