package ant.compiler.asm;

import ant.compiler.asm.Asm;
import ant.compiler.asm.Asm.*;

import java.util.Map;
import java.util.IdentityHashMap;
import java.util.ArrayList;

public class StateMinimizer {
    ArrayList commands;

    public StateMinimizer(ArrayList commands) {
	this.commands = commands;
    }

    public Asm minimize() {
	Map equals = computeEquality();

	int cnt = commands.size();
	for (int n = 0; n < cnt; n++) {
	    Asm op = (Asm) commands.get(n);

	    if (op instanceof Sequential) {
		((Sequential) op).next = (Asm) equals.get(((Sequential) op).next);
	    } else {
		((Conditional) op).nextThen = (Asm) equals.get(((Conditional) op).nextThen);
		((Conditional) op).nextElse = (Asm) equals.get(((Conditional) op).nextElse);
	    }
	}

	return (Asm) commands.get(0);
    }

    private Map computeEquality() {
	Map equals0 = new IdentityHashMap();
	Map equals1 = new IdentityHashMap();

	int cnt = commands.size();
	for (int a = 0; a < cnt; a++) {
	    Asm aOp = (Asm) commands.get(a);
	    equals1.put(aOp, aOp);

	    for (int b = 0; b < a; b++) {
		Asm bOp = (Asm) commands.get(b);

		if (aOp.equals(bOp)) {
		    equals1.put(aOp, bOp);
		    break;
		}
	    }
	}

	for (int n = 0; n < cnt; n++) {
	    Map tmp = equals0;
	    equals0 = equals1;
	    equals1 = tmp;
	    equals1.clear();

	    for (int a = 0; a < cnt; a++) {
		Asm aOp = (Asm) commands.get(a);
		equals1.put(aOp, aOp);

		Asm aRepresentative = (Asm) equals0.get(aOp);

		for (int b = aRepresentative.addr; b < a; b++) {
		    Asm bOp = (Asm) commands.get(b);

		    if (aRepresentative == equals0.get(bOp)) {
			if (aOp instanceof Sequential) {
			    if (equals0.get(((Sequential) aOp).next) == 
				equals0.get(((Sequential) bOp).next)
			    ) {
				equals1.put(aOp, bOp);
				break;
			    }
			} else {
			    if ((equals0.get(((Conditional) aOp).nextThen) == 
				    equals0.get(((Conditional) bOp).nextThen)) && 
				(equals0.get(((Conditional) aOp).nextElse) == 
				    equals0.get(((Conditional) bOp).nextElse))
			    ) {
				equals1.put(aOp, bOp);
				break;
			    }
			}
		    }
		}
	    }
	}

	return equals1;
    }
}
