/*
 * This file is part of the Ant Control Language compiler developed by
 * the team "KaNT" during the ICFP 2004 programming contest.
 *
 * ----------------------------------------------------------------------
 * 
 * Copyright (C) 2004  the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke
 * Thomas Moschny
 * Bernhard Haumacher
 * Bjoern-Oliver Hartmann
 *
 * ----------------------------------------------------------------------
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA 02111-1307, USA.
 *
 */

package ant.compiler.ast;

import ant.compiler.ast.Tree.*;


/**
 * Abstract visitor that provides default implementations for
 * visit methods.
 * 
 * <p>License: GPL</p>
 *
 * @author <a href="http://www.haumacher.de/>">Bernhard Haumacher</a>
 */
public abstract class AbstractVisitor implements Visitor {
    protected void visitTree(Tree tree) {}
    protected void visitExpression(Expression tree) {visitTree(tree); }
    protected void visitBinaryExpression(BinaryExpression tree) {visitExpression(tree); }

    public void visit(Label tree) {visitTree(tree); }
    public void visit(Block tree) {visitTree(tree); }
    public void visit(If tree) {visitTree(tree); }
    public void visit(Goto tree) {visitTree(tree); }
    public void visit(Proc tree) {visitTree(tree); }
    public void visit(Call tree) {visitTree(tree); }
    public void visit(MarkCommand tree) {visitTree(tree); }
    public void visit(UnmarkCommand tree) {visitTree(tree); }
    public void visit(TurnCommand tree) {visitTree(tree); }
    public void visit(MoveCommand tree) {visitTree(tree); }
    public void visit(PickUpCommand tree) {visitTree(tree); }
    public void visit(DropCommand tree) {visitTree(tree); }

    public void visit(Var tree) {visitTree(tree); }
    public void visit(Set tree) {visitTree(tree); }
    public void visit(Assign tree) {visitTree(tree); }
    public void visit(Test tree) {visitExpression(tree); }
    public void visit(Const tree) {visitExpression(tree); }

    public void visit(AndExpression tree) {visitBinaryExpression(tree); }
    public void visit(OrExpression tree) {visitBinaryExpression(tree); }
    public void visit(NotExpression tree) {visitExpression(tree); }
    public void visit(ConditionTest tree) {visitExpression(tree); }
    public void visit(WallTest tree) {visitExpression(tree); }
    public void visit(FoodTest tree) {visitExpression(tree); }
    public void visit(RandomTest tree) {visitExpression(tree); }
}
