/*
 * This file is part of the Ant Control Language compiler developed by
 * the team "KaNT" during the ICFP 2004 programming contest.
 *
 * ----------------------------------------------------------------------
 * 
 * Copyright (C) 2004  the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke
 * Thomas Moschny
 * Bernhard Haumacher
 * Bjoern-Oliver Hartmann
 *
 * ----------------------------------------------------------------------
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA 02111-1307, USA.
 *
 */

package ant.compiler.ast;

import ant.compiler.ast.Tree.*;


/**
 * Visitor that visits a tree fragment in depth first order.
 * 
 * <p>License: GPL</p>
 *
 * @author <a href="http://www.haumacher.de/>">Bernhard Haumacher</a>
 */
public class DFSVisitor extends AbstractVisitor {
    protected void beforeDescend(Tree tree) {}

    public void visit(Label tree) {
	super.visit(tree);

	beforeDescend(tree);
	tree.stmt.visit(this);
    }

    public void visit(Block tree) {
	super.visit(tree);

	for (int n = 0; n < tree.stmts.length; n++) {
	    beforeDescend(tree);
	    tree.stmts[n].visit(this);
	}
    }

    public void visit(Proc tree) {
	super.visit(tree);

	for (int n = 0; n < tree.params.length; n++) {
	    beforeDescend(tree);
	    tree.params[n].visit(this);
	}

	beforeDescend(tree);
	tree.body.visit(this);
    }

    public void visit(Call tree) {
	super.visit(tree);

	for (int n = 0; n < tree.args.length; n++) {
	    beforeDescend(tree);
	    tree.args[n].visit(this);
	}
    }

    public void visit(Assign tree) {
	super.visit(tree);

	beforeDescend(tree);
	tree.expr.visit(this);
    }

    public void visit(If tree) {
	super.visit(tree);

	beforeDescend(tree);
	tree.expr.visit(this);

	beforeDescend(tree);
	tree.thenStmt.visit(this);

	beforeDescend(tree);
	tree.elseStmt.visit(this);
    }

    public void visit(NotExpression tree) {
	super.visit(tree);

	beforeDescend(tree);
	tree.sub.visit(this);
    }

    public void visit(AndExpression tree) {
	super.visit(tree);

	beforeDescend(tree);
	tree.left.visit(this);

	beforeDescend(tree);
	tree.right.visit(this);
    }

    public void visit(OrExpression tree) {
	super.visit(tree);

	beforeDescend(tree);
	tree.left.visit(this);

	beforeDescend(tree);
	tree.right.visit(this);
    }
}
