/*
 * This file is part of the Ant Control Language compiler developed by
 * the team "KaNT" during the ICFP 2004 programming contest.
 *
 * ----------------------------------------------------------------------
 * 
 * Copyright (C) 2004  the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke
 * Thomas Moschny
 * Bernhard Haumacher
 * Bjoern-Oliver Hartmann
 *
 * ----------------------------------------------------------------------
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA 02111-1307, USA.
 *
 */

package ant.compiler.ast;

import ant.compiler.ast.Tree.*;


/**
 * Visitor that transforms a tree fragment. 
 * 
 * <p>License: GPL</p>
 *
 * @author <a href="http://www.haumacher.de/>">Bernhard Haumacher</a>
 */
public class Transformer extends AbstractVisitor {
    private Tree result;

    protected void afterUpdate(Tree orig, Tree result) {
    }

    private void checkForUpdate(Tree orig) {
	if (result != orig) {
	    afterUpdate(orig, result);
	}
    }

    protected void setResult(Tree result) {
	this.result = result;
    }

    public Tree getResult() {
	return this.result;
    }

    protected void visitTree(Tree tree) {
	setResult(tree);
    }

    public void visit(Label tree) {
	tree.stmt.visit(this);
	{
	    Tree orig = tree.stmt;
	    tree.stmt = getResult();
	    checkForUpdate(orig);
	}
	super.visit(tree);
    }

    public void visit(Block tree) {
	for (int n = 0; n < tree.stmts.length; n++) {
	    tree.stmts[n].visit(this);
	    {
		Tree orig = tree.stmts[n];
		tree.stmts[n] = getResult();
		checkForUpdate(orig);
	    }
	}
	super.visit(tree);
    }

    public void visit(If tree) {
	tree.expr.visit(this);
	{
	    Tree orig = tree.expr;
	    tree.expr = (Expression) getResult();
	    checkForUpdate(orig);
	}

	tree.thenStmt.visit(this);
	{
	    Tree orig = tree.thenStmt;
	    tree.thenStmt = getResult();
	    checkForUpdate(orig);
	}

	tree.elseStmt.visit(this);
	{
	    Tree orig = tree.elseStmt;
	    tree.elseStmt = getResult();
	    checkForUpdate(orig);
	}

	super.visit(tree);
    }


    public void visit(Assign tree) {
	tree.expr.visit(this);
	{
	    Tree orig = tree.expr;
	    tree.expr = (Expression) getResult();
	    checkForUpdate(orig);
	}

	super.visit(tree);
    }

    public void visit(NotExpression tree) {
	tree.sub.visit(this);
	{
	    Tree orig = tree.sub;
	    tree.sub = (Expression) getResult();
	    checkForUpdate(orig);
	}

	super.visit(tree);
    }

    public void visit(AndExpression tree) {
	tree.left.visit(this);
	{
	    Tree orig = tree.left;
	    tree.left = (Expression) getResult();
	    checkForUpdate(orig);
	}

	tree.right.visit(this);
	{
	    Tree orig = tree.right;
	    tree.right = (Expression) getResult();
	    checkForUpdate(orig);
	}

	super.visit(tree);
    }

    public void visit(OrExpression tree) {
	tree.left.visit(this);
	{
	    Tree orig = tree.left;
	    tree.left = (Expression) getResult();
	    checkForUpdate(orig);
	}

	tree.right.visit(this);
	{
	    Tree orig = tree.right;
	    tree.right = (Expression) getResult();
	    checkForUpdate(orig);
	}

	super.visit(tree);
    }
}
