/*
 * This file is part of the Ant Control Language compiler developed by
 * the team "KaNT" during the ICFP 2004 programming contest.
 *
 * ----------------------------------------------------------------------
 * 
 * Copyright (C) 2004  the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke
 * Thomas Moschny
 * Bernhard Haumacher
 * Bjoern-Oliver Hartmann
 *
 * ----------------------------------------------------------------------
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA 02111-1307, USA.
 *
 */

package ant.compiler.gen;

import ant.compiler.ast.Tree;
import ant.compiler.ast.Tree.*;
import ant.compiler.ast.AbstractVisitor;

import ant.compiler.asm.Asm;
import ant.compiler.asm.Asm.*;

import java.util.HashMap;
import java.util.ArrayList;


/**
 * {@link ant.compiler.gen.CodeGenerator Code generation} for {@link
 * ant.compiler.ast.Tree.Expression expressions}.
 *
 * <p>License: GPL</p>
 *
 * @author <a href="http://www.haumacher.de/>">Bernhard Haumacher</a>
 */
public class ExpressionEvaluator extends AbstractVisitor {
    Asm thenOp;
    Asm elseOp;

    Asm result;

    public void reset() {
	result = null;
    }

    public void setThen(Asm thenOp) {
	this.thenOp = thenOp;
    }

    public void setElse(Asm elseOp) {
	this.elseOp = elseOp;
    }

    public Asm getResult() {
	return result;
    }

    public void visit(AndExpression tree) {
	Asm thenOp = this.thenOp;
	Asm elseOp = this.elseOp;
	tree.right.visit(this);

	this.thenOp = result;
	this.elseOp = elseOp;
	tree.left.visit(this);
    }

    public void visit(OrExpression tree) {
	Asm thenOp = this.thenOp;
	Asm elseOp = this.elseOp;
	tree.right.visit(this);

	this.thenOp = thenOp;
	this.elseOp = result;
	tree.left.visit(this);
    }

    public void visit(NotExpression tree) {
	Asm thenOp = this.thenOp;
	Asm elseOp = this.elseOp;

	this.thenOp = elseOp;
	this.elseOp = thenOp;
	tree.sub.visit(this);
    }

    public void visit(ConditionTest tree) {
	result = new Sense(tree.direction, tree.condition, thenOp, elseOp);
    }

    public void visit(Const tree) {
	result = new Nop(tree.value ? thenOp : elseOp);
    }

    public void visit(WallTest tree) {
	result = new NopWallTest(elseOp, thenOp);
    }

    public void visit(FoodTest tree) {
	result = new NopFoodTest(thenOp, elseOp);
    }

    public void visit(RandomTest tree) {
	result = new Flip(tree.probability, thenOp, elseOp);
    }

    public void visit(Tree tree) {
	throw new AssertionError("evaluation of non expression");
    }
}
