/*
 * This file is part of the Ant Control Language compiler developed by
 * the team "KaNT" during the ICFP 2004 programming contest.
 *
 * ----------------------------------------------------------------------
 * 
 * Copyright (C) 2004  the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke
 * Thomas Moschny
 * Bernhard Haumacher
 * Bjoern-Oliver Hartmann
 *
 * ----------------------------------------------------------------------
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA 02111-1307, USA.
 *
 */

package ant.compiler.trafo;

import ant.compiler.ast.Tree;
import ant.compiler.ast.Tree.*;
import ant.compiler.ast.CopyVisitor;


/**
 * Visitor class transforming names in an Ant Control Program
 * fragment.
 * 
 * <p>License: GPL</p>
 *
 * @author <a href="http://www.haumacher.de/>">Bernhard Haumacher</a>
 */
public abstract class NameTransformer extends CopyVisitor {
    protected String transformNameTree(Tree tree, String name) {return name; }

    protected String transformName(Label tree) {return transformNameTree(tree, tree.name); }
    protected String transformName(Goto tree) {return transformNameTree(tree, tree.name); }
    protected String transformName(Proc tree) {return transformNameTree(tree, tree.name); }
    protected String transformName(Call tree) {return transformNameTree(tree, tree.name); }
    protected String transformName(Var tree) {return transformNameTree(tree, tree.name); }
    protected String transformName(Set tree) {return transformNameTree(tree, tree.name); }
    protected String transformName(Assign tree) {return transformNameTree(tree, tree.name); }
    protected String transformName(Test tree) {return transformNameTree(tree, tree.name); }

    public void visit(Label tree) {
	super.visit(tree);

	Label copy = (Label) getCopy();
	copy.name = transformName(tree);
    }

    public void visit(Goto tree) {
	super.visit(tree);

	Goto copy = (Goto) getCopy();
	copy.name = transformName(tree);
    }

    public void visit(Proc tree) {
	super.visit(tree);

	Proc copy = (Proc) getCopy();
	copy.name = transformName(tree);
    }

    public void visit(Call tree) {
	super.visit(tree);

	Call copy = (Call) getCopy();
	copy.name = transformName(tree);
    }

    public void visit(Var tree) {
	super.visit(tree);

	Var copy = (Var) getCopy();
	copy.name = transformName(tree);
    }

    public void visit(Set tree) {
	super.visit(tree);

	Set copy = (Set) getCopy();
	copy.name = transformName(tree);
    }

    public void visit(Assign tree) {
	super.visit(tree);

	Assign copy = (Assign) getCopy();
	copy.name = transformName(tree);
    }

    public void visit(Test tree) {
	super.visit(tree);

	Test copy = (Test) getCopy();
	copy.name = transformName(tree);
    }
}
