/*
 * This file is part of the Ant Control Language compiler developed by
 * the team "KaNT" during the ICFP 2004 programming contest.
 *
 * ----------------------------------------------------------------------
 * 
 * Copyright (C) 2004  the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke
 * Thomas Moschny
 * Bernhard Haumacher
 * Bjoern-Oliver Hartmann
 *
 * ----------------------------------------------------------------------
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA 02111-1307, USA.
 *
 */

package ant.compiler.util;

import ant.compiler.ast.Tree.*;
import ant.compiler.ast.TurnDirection;
import ant.compiler.ast.SenseDirection;
import ant.compiler.ast.Condition;
import ant.compiler.asm.Asm;
import ant.compiler.asm.AsmPrinter;


import java.io.StringWriter;
import java.io.PrintWriter;

/**
 * Utility class for printing constants of the {@link Asm} assembler.
 * 
 * <p>License: GPL</p>
 *
 * @author <a href="http://www.haumacher.de/>">Bernhard Haumacher</a>
 */
public class Printer {
    public static  String getLabel(TurnDirection direction) {
	switch(direction.id) {
	case TurnDirection.ID_LEFT: return "Left";
	case TurnDirection.ID_RIGHT: return "Right";
	default: throw new AssertionError("unknown direction");
	}
    }

    public static String getLabel(SenseDirection direction) {
	switch(direction.id) {
	case SenseDirection.ID_HERE: return "Here";
	case SenseDirection.ID_AHEAD: return "Ahead";
	case SenseDirection.ID_LEFT_AHEAD: return "LeftAhead";
	case SenseDirection.ID_RIGHT_AHEAD: return "RightAhead";
	default: throw new AssertionError("unknown direction");
	}
    }

    public static String getLabel(Condition condition) {
	switch(condition.id) {
	case Condition.ID_FRIEND: return "Friend";
	case Condition.ID_FOE: return "Foe";
	case Condition.ID_FRIEND_WITH_FOOD: return "FriendWithFood";
	case Condition.ID_FOE_WITH_FOOD: return "FoeWithFood";
	case Condition.ID_FOOD: return "Food";
	case Condition.ID_ROCK: return "Rock";
	case Condition.ID_MARKER_0: return "Marker";
	case Condition.ID_MARKER_1: return "Marker";
	case Condition.ID_MARKER_2: return "Marker";
	case Condition.ID_MARKER_3: return "Marker";
	case Condition.ID_MARKER_4: return "Marker";
	case Condition.ID_MARKER_5: return "Marker";
	case Condition.ID_FOE_MARKER: return "FoeMarker"; 
	case Condition.ID_HOME: return "Home";
	case Condition.ID_FOE_HOME: return "FoeHome";
	default: throw new AssertionError("unknown direction");
	}
    }

    public static boolean hasMarker(Condition condition) {
	return
	    (condition.id >= Condition.ID_MARKER_0) &&
	    (condition.id <= Condition.ID_MARKER_5);
    }

    public static int getMarker(Condition condition) {
	if (! hasMarker(condition)) {
	    throw new AssertionError("no marker in condition: " + getLabel(condition));
	}

	return condition.id - Condition.ID_MARKER_0;
    }

    public static String debugToString(Asm op) {
	StringWriter s = new StringWriter();
	PrintWriter p = new PrintWriter(s);
	op.visit(new AsmPrinter(null, p, /*debug*/ false));
	p.flush();
	return  s.toString().trim();
    }
}
