/*
 * ANT Simulation Programm
 *
 * ----------------------------------------------------------------------
 * 
 * (c) by the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke <hauke@ipd.uka.de>
 * Thomas Moschny <moschny@ipd.uka.de>
 * Bernhard Haumacher <haui@haumacher.de>
 * Bjoern Hartmann <bjoern@derhartmann.de>
 *
 * ----------------------------------------------------------------------
 * 
 * This is free software (GPL).
 * 
 * $Rev: 1541 $
 * $Date: 2004-06-05 21:07:02 +0200 (Sat, 05 Jun 2004) $
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include "sim.h"

#include <popt.h>

char *arg_p1 = "none";
char *arg_p2 = "none";
char *arg_world = "none";
int arg_steps = 200;
int arg_fastforward = 1;

void usage(poptContext optCon, int exitcode, char *error, char *addl)
{
    poptPrintUsage(optCon, stderr, 0);
    if (error) fprintf(stderr, "%s: %s\n", error, addl);
    exit(exitcode);
}

void parse_opt(int argc, char **argv)
{
    char    c;            /* used for argument parsing */
    poptContext optCon;   /* context for parsing command-line options */
    
    struct poptOption optionsTable[] = {
	{ "p1"  , '1', POPT_ARG_STRING, &arg_p1 , 0,
	  "apc 1", "name" },
	{ "p2"  , '2', POPT_ARG_STRING, &arg_p2 , 0,
	  "apc 2", "name" },
	{ "world"  , 'w', POPT_ARG_STRING, &arg_world , 0,
	  "worldname", "name" },
	{ "steps"  , 's', POPT_ARG_INT, &arg_steps , 0,
	  "steps", "num" },
	{ "ff"  , 'f', POPT_ARG_INT, &arg_fastforward , 0,
	  "fast_forward", "num" },
/*	{ "flag" , 'f', POPT_ARGFLAG_OR, &arg_flag, 0,
	  "flag description", "" },*/
	POPT_AUTOHELP
	{ NULL, 0, 0, NULL, 0, NULL, NULL }
    };
    
    optCon = poptGetContext(NULL, argc,(const char **) argv, optionsTable, 0);

    if (argc < 2) {
	poptPrintUsage(optCon, stderr, 0);
	exit(1);
    }
    
    /* Now do options processing, get portname */
    while ((c = poptGetNextOpt(optCon)) >= 0) {
    }
    if (c < -1) {
	/* an error occurred during option processing */
	fprintf(stderr, "%s: %s\n",
		poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
		poptStrerror(c));
	poptPrintHelp(optCon, stderr, 0);
	exit(1);
    }
    
    poptFreeContext(optCon);
}

int main(int argc, char **argv)
{
    antprog_t *ap1, *ap2;
    world_t *world;
    ratio_t ratio;

    
    parse_opt(argc, argv);

    num_simsteps = arg_steps;
    num_dumpsteps = arg_fastforward;
    ap1 = antprog_read(arg_p1, arg_p1);
    ap2 = antprog_read(arg_p2, arg_p2);

    world = world_read(arg_world);

//    printf("# Using map\n");
    printf("%s\n", arg_world);
    ratio = simulate(ap1, ap2, world, DUMP_DEBUG);

//    printf("%d %d\n", ratio.red, ratio.black);
    
    return 0;
}


