/*
 * ANT Simulation Programm
 *
 * ----------------------------------------------------------------------
 * 
 * (c) by the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke <hauke@ipd.uka.de>
 * Thomas Moschny <moschny@ipd.uka.de>
 * Bernhard Haumacher <haui@haumacher.de>
 * Bjoern Hartmann <bjoern@derhartmann.de>
 *
 * ----------------------------------------------------------------------
 * 
 * This is free software (GPL).
 * 
 * $Rev: 1638 $
 * $Date: 2004-06-07 16:38:38 +0200 (Mon, 07 Jun 2004) $
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include <sys/types.h>
#include <unistd.h>
#include <assert.h>
#include <time.h>
#include "sim.h"

#include <popt.h>

int arg_verbose=0;
char *arg_p1 = "none";
char *arg_world = "none";
char *arg_outname = NULL;
int arg_steps = 100000;
int arg_generations = 10;
int arg_mods = 3;

void usage(poptContext optCon, int exitcode, char *error, char *addl)
{
    poptPrintUsage(optCon, stderr, 0);
    if (error) fprintf(stderr, "%s: %s\n", error, addl);
    exit(exitcode);
}

void parse_opt(int argc, char **argv)
{
    char    c;            /* used for argument parsing */
    poptContext optCon;   /* context for parsing command-line options */
    
    struct poptOption optionsTable[] = {
	{ "verbose"  , 'v', POPT_ARG_INT, &arg_verbose , 0,
	  "be more verbose", "level" },
	{ "p1"  , '1', POPT_ARG_STRING, &arg_p1 , 0,
	  "initial apc", "name" },
	{ "out"  , 'o', POPT_ARG_STRING, &arg_outname , 0,
	  "out apc", "name" },
	{ "world"  , 'w', POPT_ARG_STRING, &arg_world , 0,
	  "worldname", "name" },
	{ "steps"  , 's', POPT_ARG_INT, &arg_steps , 0,
	  "steps", "num" },
	{ "gens"  , 'g' , POPT_ARG_INT, &arg_generations , 0,
	  "num individuals", "num" },
	{ "mods"  , 'm' , POPT_ARG_INT, &arg_mods , 0,
	  "modifications in mutate", "num" },

	
/*	{ "flag" , 'f', POPT_ARGFLAG_OR, &arg_flag, 0,
	  "flag description", "" },*/
	POPT_AUTOHELP
	{ NULL, 0, 0, NULL, 0, NULL, NULL }
    };
    
    optCon = poptGetContext(NULL, argc,(const char **) argv, optionsTable, 0);

    if (argc < 2) {
	poptPrintUsage(optCon, stderr, 0);
	exit(1);
    }
    
    /* Now do options processing, get portname */
    while ((c = poptGetNextOpt(optCon)) >= 0) {
    }
    if (c < -1) {
	/* an error occurred during option processing */
	fprintf(stderr, "%s: %s\n",
		poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
		poptStrerror(c));
	poptPrintHelp(optCon, stderr, 0);
	exit(1);
    }
    
    poptFreeContext(optCon);
}

static
void expr_random(antexpr_t *expr, int codelen)
{
    int token = rand() % TOKEN_CNT;

    if (token == TOKEN_ILLEGAL) {
	/* Have luck - no mutation */
	return;
    }

    expr->token = token;
    switch (token) {
    case TOKEN_SENSE:
	expr->u.sense.sensedir = rand() % SDIR_CNT;
	expr->u.sense.st1 = rand() % codelen;
	expr->u.sense.st2 = rand() % codelen;
	expr->u.sense.cond = rand() % COND_CNT;
	expr->u.sense.i = rand() % 6;
	break;

    case TOKEN_MARK:
	expr->u.mark.i = rand() % 6;
	expr->u.mark.st = rand() % codelen;
	break;

    case TOKEN_UNMARK:
	expr->u.unmark.i = rand() % 6;
	expr->u.unmark.st = rand() % codelen;
	break;
	
    case TOKEN_PICKUP:
	expr->u.pickup.st1 = rand() % codelen;
	expr->u.pickup.st2 = rand() % codelen;
	break;
	
    case TOKEN_DROP:
	expr->u.drop.st = rand() % codelen;
	break;
	
    case TOKEN_TURN:
	expr->u.turn.lr = (rand() % 2 == 0) ? TURN_LEFT : TURN_RIGHT;
	expr->u.turn.st = rand() % codelen;
	break;
	
    case TOKEN_MOVE:
	expr->u.move.st1 = rand() % codelen;
	expr->u.move.st2 = rand() % codelen;
	break;

    case TOKEN_FLIP:
	expr->u.flip.p = (rand() % 20) + 1;
	expr->u.flip.st1 = rand() % codelen;
	expr->u.flip.st2 = rand() % codelen;
	break;
    default:
	printf("Internal error in %s : %d\n", __func__, __LINE__);
	exit(1);
    }
}

static
void mutate(antprog_t *dest, antprog_t *src)
{
    int i;

    assert(dest->proglength == src->proglength);

    for (i = 0; i < dest->proglength; i++) {
	dest->expr[i] = src->expr[i];
    }

    for (i = 0; i < arg_mods; i++) {
	int expi = rand() % dest->proglength;
	expr_random(&dest->expr[expi], dest->proglength);
    }
}

static
void mutate2(antprog_t *dest, antprog_t *src1, antprog_t *src2)
{
    int i;
    int breakpoint;

    assert(dest->proglength == src1->proglength);
    assert(dest->proglength == src2->proglength);
    
    breakpoint = rand() % dest->proglength;
    
    for (i = 0; i < breakpoint; i++) {
	dest->expr[i] = src1->expr[i];
    }

    for (i = breakpoint; i < dest->proglength; i++) {
	dest->expr[i] = src2->expr[i];
    }
}



antprog_t **antprogs;
char filename[1000]; /* yeah i love static arrays */

int main(int argc, char **argv)
{
    world_t *world;
    ratio_t ratio;
    int i;
    int ant_version = 0;

    srand(getpid() + time(NULL));
    
    parse_opt(argc, argv);

    if (!arg_outname) arg_outname = arg_p1;
    
    num_simsteps = arg_steps;
    antprogs = malloc(sizeof(*antprogs) * arg_generations);

    world = world_read(arg_world);
    if (!world) {
	perror("cant find world!");
	exit(1);
    }
    

    printf("Generations: %d\n", arg_generations);

    for (i = 0; i < arg_generations; i++) {
	/* slow but not timecritical */
	antprogs[i] = antprog_read(arg_p1, arg_p1);
    }

    while (1) {

	/* create a new one by split-merge (crossover) (the old
	   worsted gets overwritten

	   args are (dest, src1, src2 ) */
	mutate2(antprogs[arg_generations - 1],
	       antprogs[rand() % (arg_generations - 1)],
	       antprogs[rand() % (arg_generations - 1)]);

	/* mutate the new one a bit */
	mutate(antprogs[arg_generations - 1], antprogs[arg_generations - 1]);
	

	for (i = arg_generations - 1; i > 0; i--) {
	    ratio_t r;
	    antprog_t *tmp;

	    r = simulate2(antprogs[i], antprogs[i - 1], world, DUMP_NONE);

	    printf("Contest ant %3d vs. %3d (%4d : %4d)\n",
		   i, i - 1, r.red, r.black);
	    fflush(stdout);
	    if (r.red <= r.black) goto out; /* next round */

	    /* exchange i and i-1 */
	    tmp = antprogs[i]; antprogs[i] = antprogs[i - 1]; antprogs[i - 1] = tmp;
	}

	sprintf(filename, "%s-%d", arg_outname, ant_version);
	printf("We have a new leader %s!\n", filename);
	antprog_write(filename, antprogs[0]);
	ant_version++;
    out:
	{}
    }
    
    return 0;
}

/*
  Local Variables:
  c-basic-offset: 4
  End:
*/
