/*
 * ANT Simulation Programm
 *
 * ----------------------------------------------------------------------
 * 
 * (c) by the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke <hauke@ipd.uka.de>
 * Thomas Moschny <moschny@ipd.uka.de>
 * Bernhard Haumacher <haui@haumacher.de>
 * Bjoern Hartmann <bjoern@derhartmann.de>
 *
 * ----------------------------------------------------------------------
 * 
 * This is free software (GPL).
 * 
 * $Rev: 1632 $
 * $Date: 2004-06-07 14:35:26 +0200 (Mon, 07 Jun 2004) $
 *
 * Contest control programm.
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include "sim.h"

#include <popt.h>

int arg_verbose=0;
char *arg_p1 = "none";
char *arg_p2 = "none";
char *arg_world = "none";
int arg_steps = 100000;
int arg_direction = 0; /* default is both */
int arg_count = 0;

void usage(poptContext optCon, int exitcode, char *error, char *addl)
{
    poptPrintUsage(optCon, stderr, 0);
    if (error) fprintf(stderr, "%s: %s\n", error, addl);
    exit(exitcode);
}

void parse_opt(int argc, char **argv)
{
    char    c;            /* used for argument parsing */
    poptContext optCon;   /* context for parsing command-line options */
    
    struct poptOption optionsTable[] = {
	{ "verbose"  , 'v', POPT_ARG_INT, &arg_verbose , 0,
	  "be more verbose", "level" },
	{ "p1 red"  , '1', POPT_ARG_STRING, &arg_p1 , 0,
	  "apc 1", "name" },
	{ "p2 black"  , '2', POPT_ARG_STRING, &arg_p2 , 0,
	  "apc 2", "name" },
	{ "world"  , 'w', POPT_ARG_STRING, &arg_world , 0,
	  "worldname", "name" },
	{ "steps"  , 's', POPT_ARG_INT, &arg_steps , 0,
	  "steps", "num" },
	{ "direction"  , 'd', POPT_ARG_INT, &arg_direction, 0,
	  "run only p1vs.p2(1) or p2vs.p1(2) default both", "1 or 2(reverse)" },
	{ "count"  , 'c', 0, &arg_count, 0,
	  "count the ants", "xxx" },
	 /* default is both */
/*	{ "flag" , 'f', POPT_ARGFLAG_OR, &arg_flag, 0,
	  "flag description", "" },*/
	POPT_AUTOHELP
	{ NULL, 0, 0, NULL, 0, NULL, NULL }
    };
    
    optCon = poptGetContext(NULL, argc,(const char **) argv, optionsTable, 0);

    if (argc < 2) {
	poptPrintUsage(optCon, stderr, 0);
	exit(1);
    }
    
    /* Now do options processing, get portname */
    while ((c = poptGetNextOpt(optCon)) >= 0) {
    }
    if (c < -1) {
	/* an error occurred during option processing */
	fprintf(stderr, "%s: %s\n",
		poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
		poptStrerror(c));
	poptPrintHelp(optCon, stderr, 0);
	exit(1);
    }
    
    poptFreeContext(optCon);
}

int main(int argc, char **argv)
{
    antprog_t *ap1, *ap2;
    world_t *world;
    ratio_t ratio;
    ratio_t _alive;
    ratio_t *alive = NULL;
    
    parse_opt(argc, argv);

    if (arg_count) {
	alive = &_alive;
    }
    num_simsteps = arg_steps;
    ap1 = antprog_read(arg_p1, arg_p1);
    ap2 = antprog_read(arg_p2, arg_p2);

    world = world_read(arg_world);


    if (arg_direction == 0) {
	ratio = simulate2_alive(ap1, ap2, world, arg_verbose, alive);
    } else if (arg_direction == 1) {
	ratio = simulate_alive(ap1, ap2, world, arg_verbose, alive);
    } else if (arg_direction == 2) {
	int tmp;
	ratio = simulate_alive(ap2, ap1, world, arg_verbose, alive);
	tmp = ratio.red; ratio.red = ratio.black; ratio.black = tmp;
	if (alive) {
	    tmp = alive->red; alive->red = alive->black; alive->black = tmp;
	}
    } else {
	printf("--direction must be\n"
	       "  0 : both directions added\n"
	       "  1 : p1 (red) vs. p2 (black)"
	       "  2 : p2 (red) vs. p1 (black)");
	exit (1);
    }

    if (!alive) {
	printf("%d %d\n", ratio.red, ratio.black);
    } else {
	printf("%d %d alive %d %d\n",
	       ratio.red, ratio.black,
	       alive->red, alive->black);
    }
    
    return 0;
}


