/*
 * ANT Simulation Programm
 *
 * ----------------------------------------------------------------------
 * 
 * (c) by the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke <hauke@ipd.uka.de>
 * Thomas Moschny <moschny@ipd.uka.de>
 * Bernhard Haumacher <haui@haumacher.de>
 * Bjoern Hartmann <bjoern@derhartmann.de>
 *
 * ----------------------------------------------------------------------
 * 
 * This is free software (GPL).
 * 
 * $Rev: 1632 $
 * $Date: 2004-06-07 14:35:26 +0200 (Mon, 07 Jun 2004) $
 *
 * Simulator interface.
 *
 */
#ifndef _SIM_H_
#define _SIM_H_

#define TOKEN_ILLEGAL	0
#define TOKEN_SENSE	1
#define TOKEN_MARK	2
#define TOKEN_UNMARK	3
#define TOKEN_PICKUP	4
#define TOKEN_DROP	5
#define TOKEN_TURN	6
#define TOKEN_MOVE	7
#define TOKEN_FLIP	8
#define TOKEN_CNT	9

#define TURN_LEFT 5
#define TURN_RIGHT 1

#define SDIR_HERE	0
#define SDIR_AHEAD	1
#define SDIR_LEFTAHEAD	2
#define SDIR_RIGHTAHEAD	3
#define SDIR_CNT 4

#define COND_FRIEND	0
#define COND_FOE	1
#define COND_FRIENDFOOD	2
#define COND_FOEFOOD	3
#define COND_FOOD	4
#define COND_ROCK	5
#define COND_MARKER	6
#define COND_FOEMARKER	7
#define COND_HOME	8
#define COND_FOEHOME	9
#define COND_CNT 10


#define DUMP_NONE 0
#define DUMP_STD  1
#define DUMP_DEBUG 2

typedef struct pos_s {
    int x;
    int y;
} pos_t;

typedef struct antexpr_s {
    int token; /* one of TOKE_xxx */
    union {
	struct { int sensedir; int st1, st2; int cond; int i; } sense;
	struct { int i; int st; } mark;
	struct { int i; int st; } unmark;
	struct { int st1, st2; } pickup;
	struct { int st; } drop;
	struct { int lr; int st; } turn;
	struct { int st1, st2; } move;
	struct { int p; int st1, st2; } flip;
	struct { int _align_[7]; } _align_;
    } u;
} antexpr_t;

#define N_ANTEXPR 10000

typedef struct antprog_s {
    antexpr_t expr[N_ANTEXPR];
    char *name;
    int proglength;
} antprog_t;

typedef struct antstat_s {
    antprog_t		*prog; /* define implicit the color */
    unsigned int	state; /* (like programcounter initialized with 0) */
    pos_t		pos;
    unsigned int	dir; /* direction 0 - 5 */
    int			color; /* 0 = red, 1 = black */
    unsigned int	resting;
    unsigned int	has_food;
    unsigned int	is_dead; /* !alive */
    unsigned int	id; /* unique id is index in ants_t array */
} antstat_t;

#define MAX_WORLD_X 128
#define MAX_WORLD_Y 128

#define ANT_RED 0
#define ANT_BLACK 1



#define NO_ANT -1

typedef struct field_s {
    unsigned int food;
    unsigned int rocky;
    unsigned int markers[2]; /* for red and black markers */
    unsigned int home[2]; /* boolean: is home of ant red or black */
    int ant_id; /* -1 is NO_ANT */
    int original_food;
} field_t;

typedef struct world_s {
    field_t field[MAX_WORLD_Y][MAX_WORLD_X];
    int dimx, dimy;
} world_t;

#define MAX_ANTS 200

typedef struct ants_s {
    antstat_t	ant[MAX_ANTS];
    int num_ants;
} ants_t;

typedef struct ratio_s {
    int red;  /* foot in red anthill */
    int black; /* foot in black anthill */
} ratio_t;

extern int num_simsteps;
extern int num_dumpsteps;

ratio_t simulate_alive(antprog_t *ant_red, antprog_t *ant_black, world_t *_world, int dump,
		 ratio_t *alive);
ratio_t simulate2_alive(antprog_t *ant_red, antprog_t *ant_black, world_t *_world, int dump,
		  ratio_t *alive);

/* backward compat */
static inline
ratio_t simulate(antprog_t *ant_red, antprog_t *ant_black, world_t *_world, int dump)
{
    return simulate_alive(ant_red, ant_black, _world, dump, NULL);
}

static inline
ratio_t simulate2(antprog_t *ant_red, antprog_t *ant_black, world_t *_world, int dump)
{
    return simulate2_alive(ant_red, ant_black, _world, dump, NULL);
}


void antprog_free(antprog_t *antp);
void antprog_setname(antprog_t *antp, char *name);
antprog_t *antprog_alloc(char *name);

world_t *world_alloc(void);
void world_free(world_t *world);

/* from sim_io.c */

antprog_t *antprog_read(char *filename, char *comment);
world_t *world_read(char *filename);
void antprog_write(char *filename, antprog_t *prog);

#endif

/*
  Local Variables:
  c-basic-offset: 4
  End:
*/
