/*
 * ANT Simulation Programm
 *
 * ----------------------------------------------------------------------
 * 
 * (c) by the ICFP 2004 Team "KaNT"
 *
 * Jens Hauke <hauke@ipd.uka.de>
 * Thomas Moschny <moschny@ipd.uka.de>
 * Bernhard Haumacher <haui@haumacher.de>
 * Bjoern Hartmann <bjoern@derhartmann.de>
 *
 * ----------------------------------------------------------------------
 * 
 * This is free software (GPL).
 * 
 * $Rev: 1519 $
 * $Date: 2004-06-05 15:46:19 +0200 (Sat, 05 Jun 2004) $
 *
 * Simulator I/O routines.
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"


static
int tok_to_sdir(char *token)
{
    if        (!strcasecmp(token, "here")) {
	return SDIR_HERE;
    } else if (!strcasecmp(token, "ahead")) {
	return SDIR_AHEAD;
    } else if (!strcasecmp(token, "leftahead")) {
	return SDIR_LEFTAHEAD;
    } else if (!strcasecmp(token, "rightahead")) {
	return SDIR_RIGHTAHEAD;
    } else {
	printf("Warning: unkown token (%s) expecting dir\n", token);
	return SDIR_HERE;
    }
}

static
char* sdir_to_tok(int token)
{
    switch (token){
	
    case SDIR_HERE:
	return "Here";
	
    case SDIR_AHEAD:
	return "Ahead";
     
    case SDIR_LEFTAHEAD:
	return "Leftahead";
	
    case SDIR_RIGHTAHEAD:
	return "Rightahead";

    default:
	return "<unknown>";
    }
}


static
int tok_to_cond(char *token)
{
    if        (!strcasecmp(token, "friend")) {
	return COND_FRIEND;
    } else if (!strcasecmp(token, "foe")) {
	return COND_FOE;
    } else if (!strcasecmp(token, "friendwithfood")) {
	return COND_FRIENDFOOD;
    } else if (!strcasecmp(token, "foewithfood")) {
	return COND_FOEFOOD;
    } else if (!strcasecmp(token, "food")) {
	return COND_FOOD;
    } else if (!strcasecmp(token, "rock")) {
	return COND_ROCK;
    } else if (!strcasecmp(token, "marker")) {
	return COND_MARKER;
    } else if (!strcasecmp(token, "foemarker")) {
	return COND_FOEMARKER;
    } else if (!strcasecmp(token, "home")) {
	return COND_HOME;
    } else if (!strcasecmp(token, "foehome")) {
	return COND_FOEHOME;
    } else {
	printf("Warning: unkown token (%s) expecting cond\n", token);
	return COND_FRIEND;
    }
}

static
char* cond_to_tok(int token)
{
    switch(token){
    case COND_FRIEND:
	return "Friend";
    case COND_FOE:
	return "Foe";
    case COND_FRIENDFOOD:
	return "FriendWithFood";
    case COND_FOEFOOD:
	return "FoeWithFood";
    case COND_FOOD:
	return "Food";
    case COND_ROCK:
	return "Rock";
    case COND_MARKER:
	return "Marker";
    case COND_FOEMARKER:
	return "Foemarker";
    case COND_HOME:
	return "Home";
    case COND_FOEHOME:
	return "Foehome";
    default:
	return "<unknown>";
    }
}


static
int tok_to_turn(char *token)
{
    if        (!strcasecmp(token, "left")) {
	return TURN_LEFT;
    } else if (!strcasecmp(token, "right")) {
	return TURN_RIGHT;
    } else {
	printf("Warning: unkown token (%s) expecting turn\n", token);
	return TURN_LEFT;
    }
}

static
char* turn_to_tok(int token)
{
    switch(token){
    case TURN_LEFT:
	return "Left";
    case TURN_RIGHT:
	return "Right";
    }
    return "Left";
}


void antprog_write(char *filename, antprog_t *prog)
{
    int state;
    FILE *io;

    io = fopen(filename, "w+");

    if (!io) {
	perror("antprog_write");
	return;
    }

    for (state = 0; state < prog->proglength; state++){
	antexpr_t expr = prog->expr[state];
	switch (expr.token){
	case TOKEN_ILLEGAL:
	    fprintf(io,"<Illegal>");
	    break;

	case TOKEN_SENSE:
	    if (expr.u.sense.cond == COND_MARKER) {
		fprintf(io,"Sense %s %d %d %s %d",
		       sdir_to_tok(expr.u.sense.sensedir),
		       expr.u.sense.st1, expr.u.sense.st2,
		       cond_to_tok(COND_MARKER), expr.u.sense.i);
	    } else {
		fprintf(io,"Sense %s %d %d %s",
		       sdir_to_tok(expr.u.sense.sensedir),
		       expr.u.sense.st1, expr.u.sense.st2,
		       cond_to_tok(expr.u.sense.cond));
	    }
	    break;
	    
	case TOKEN_MARK:
	    fprintf(io,"Mark %d %d", expr.u.mark.i, expr.u.mark.st);
	    break;
	    
	case TOKEN_UNMARK:
	    fprintf(io,"Unmark %d %d", expr.u.unmark.i, expr.u.unmark.st);
	    break;
	    
	case TOKEN_PICKUP:
	    fprintf(io,"PickUp %d %d", expr.u.pickup.st1, expr.u.pickup.st2);
	    break;
	    
	case TOKEN_DROP:
	    fprintf(io,"Drop %d", expr.u.drop.st);
	    break;
	    
	case TOKEN_TURN:
	    fprintf(io,"Turn %s %d", turn_to_tok(expr.u.turn.lr), expr.u.turn.st);
	    break;
	    
	case TOKEN_MOVE:
	    fprintf(io,"Move %d %d", expr.u.move.st1, expr.u.move.st2);
	    break;
	    
	case TOKEN_FLIP:
	    fprintf(io,"Flip %d %d %d", expr.u.flip.p, expr.u.flip.st1,
		   expr.u.flip.st2);
	    break;
	    
	default:
	    fprintf(io,"Warning: illegal token in state %d.\n", state);
	}

	/* fprintf(io, " ; state %d\n", state); */
	fprintf(io, "\n");
    }
    fclose(io);
}



antprog_t *antprog_read(char *filename, char *comment)
{
    antprog_t *prog;
    char line[1000];
    int linecnt = 0;
    int state = 0;
    int cnt;
    FILE *io;
    char s1[50], s2[50];
    int i1, i2, i3, i4;

    io = fopen(filename, "r");

    if (!io) return NULL;

    prog = antprog_alloc(comment);

    while (!feof(io)) {
	char token[100];
	char *l;

	if (!fgets(line, sizeof(line), io)) break;
	
	linecnt++;
	
	sscanf(line, "%s", token);

	l = line + strlen(token);
	
	if        (!strcasecmp(token, "sense")) {
	    cnt = sscanf(l, "%s %d %d %s %d",
			 s1, &i1, &i2, s2, &i3);
	    if (cnt < 4) goto err;

	    prog->expr[state].token = TOKEN_SENSE;
	    prog->expr[state].u.sense.sensedir = tok_to_sdir(s1);
	    prog->expr[state].u.sense.st1 = i1;
	    prog->expr[state].u.sense.st2 = i2;
	    prog->expr[state].u.sense.cond = tok_to_cond(s2);
	    if (prog->expr[state].u.sense.cond == COND_MARKER) {
		prog->expr[state].u.sense.i = i3;
	    }
	    state++;
	} else if (!strcasecmp(token, "mark")) {
	    cnt = sscanf(l, "%d %d", &i1, &i2);
	    if (cnt < 2) goto err;
	    prog->expr[state].token = TOKEN_MARK;
	    prog->expr[state].u.mark.i = i1;
	    prog->expr[state].u.mark.st = i2;
	    state++;
	} else if (!strcasecmp(token, "unmark")) {
	    cnt = sscanf(l, "%d %d", &i1, &i2);
	    if (cnt < 2) goto err;
	    prog->expr[state].token = TOKEN_UNMARK;
	    prog->expr[state].u.unmark.i = i1;
	    prog->expr[state].u.unmark.st = i2;
	    state++;
	} else if (!strcasecmp(token, "pickup")) {
	    cnt = sscanf(l, "%d %d", &i1, &i2);
	    if (cnt < 2) goto err;
	    prog->expr[state].token = TOKEN_PICKUP;
	    prog->expr[state].u.pickup.st1 = i1;
	    prog->expr[state].u.pickup.st2 = i2;
	    state++;
	} else if (!strcasecmp(token, "drop")) {
	    cnt = sscanf(l, "%d", &i1);
	    if (cnt < 1) goto err;
	    prog->expr[state].token = TOKEN_DROP;
	    prog->expr[state].u.drop.st = i1;
	    state++;
	} else if (!strcasecmp(token, "turn")) {
	    cnt = sscanf(l, "%s %d", s1, &i1);
	    if (cnt < 2) goto err;
	    prog->expr[state].token = TOKEN_TURN;
	    prog->expr[state].u.turn.lr = tok_to_turn(s1);
	    prog->expr[state].u.turn.st = i1;
	    state++;
	} else if (!strcasecmp(token, "move")) {
	    cnt = sscanf(l, "%d %d", &i1, &i2);
	    if (cnt < 2) goto err;
	    prog->expr[state].token = TOKEN_MOVE;
	    prog->expr[state].u.move.st1 = i1;
	    prog->expr[state].u.move.st2 = i2;
	    state++;
	} else if (!strcasecmp(token, "flip")) {
	    cnt = sscanf(l, "%d %d %d", &i1, &i2, &i3);
	    if (cnt < 3) goto err;
	    prog->expr[state].token = TOKEN_FLIP;
	    prog->expr[state].u.flip.p = i1;
	    prog->expr[state].u.flip.st1 = i2;
	    prog->expr[state].u.flip.st2 = i3;
	    state++;
	} else {
	err:
	    fprintf(stderr, "Warning: illegal line %d in %s\n%s\n",
		    linecnt, filename, line);
	}
    }

    fclose(io);

    prog->proglength = state;
//    fprintf(stderr,"proglength %d\n", prog->proglength);
    

    return prog;
}

world_t *world_read(char *filename)
{
    world_t *world;

    FILE *io;
    char buf[100];
    int x, y;
    
    io = fopen(filename, "r");

    if (!io) return NULL;

    world = world_alloc();

    fscanf(io, "%d %d", &world->dimx, &world->dimy);

    if (world->dimx >= MAX_WORLD_X) {
	printf("dim x of world %s to big (%d) \n",
	       filename, world->dimx);
    }
    if (world->dimy >= MAX_WORLD_Y) {
	printf("dim y of world %s to big (%d) \n",
	       filename, world->dimy);
    }
    
    for (y = 0; y < world->dimy; y++) {
	for (x = 0; x < world->dimx; x++) {
	    fscanf(io, "%s", buf);

	    switch (buf[0]) {
	    case '0':
	    case '1':
	    case '2':
	    case '3':
	    case '4':
	    case '5':
	    case '6':
	    case '7':
	    case '8':
	    case '9':
		world->field[y][x].original_food = atoi(buf);
		break;
	    case '.':
		break;
	    case '+':
		world->field[y][x].home[ANT_RED] = 1;
		break;
	    case '-':
		world->field[y][x].home[ANT_BLACK] = 1;
		break;
	    case '#':
		world->field[y][x].rocky = 1;
		break;
	    default:
		printf("illegal char in world %s\n", buf);
	    }
	}
    }

    return world;
}
    
/*
  Local Variables:
  c-basic-offset: 4
  End:
*/
